# -*- coding: utf-8 -*-

from cliez.component import Component

try:
    from shlex import quote as shell_quote
except ImportError:
    from pipes import quote as shell_quote


class CheckComponent(Component):
    # un-comment this can disable global options
    # exclude_global_option = True

    def run(self, options):
        """
        plan feature

        we should check

        * only support centos or macosx
        * check pssh is installed

        :param options:
        :return:
        """

        pass

    pass



# -*- coding: utf-8 -*-

import json
import os

from cliez.component import Component

from cabric.dns.dnspod import DNSPod
from cabric.utils import (get_roots, run, bind_hosts, execute,
                          get_platform, put, fabric_settings, env, get_home,
                          current_machine,
                          mirror_put)

try:
    from shlex import quote as shell_quote
except ImportError:
    from pipes import quote as shell_quote


class ConfigComponent(Component):
    def enable_services(self, services):
        """
        :param list services: service list
        :return:
        """
        if get_platform() == 'centos':
            # run('systemctl disable %s' % ' '.join(services))
            [run('systemctl enable %s' % v) for v in services]
        else:
            self.warn("not support platform.no services enabled.")
        pass

    def reload(self, reloads, all_services):
        if all_services and reloads:
            services = [v for v in reloads if v in all_services]
            run('systemctl reload %s' % ' '.join(services))
        pass

    def start(self, services, all_services):
        if all_services and services:
            s = [v for v in services if v in all_services]
            run('systemctl start %s' % ' '.join(s))
        pass

    def stop(self, services, all_services):
        if all_services and services:
            s = [v for v in services if v in all_services]
            run('systemctl stop %s' % ' '.join(s))
        pass

    def restart(self, restarts, all_services):

        if all_services and restarts:
            services = [v for v in restarts if v in all_services]
            if services:
                run('systemctl restart %s' % ' '.join(services))
            else:
                self.warn(
                    "restart progress actived,but no restart service found.")
        pass

    def upload_config_file(self, config_root, working_env, stages=[]):
        """
        support upload other config or project cabric files
        :param config_root:
        :param working_env:
        :param stages:
        :return:
        """

        stages.append(working_env)

        upload_roots = map(
            lambda x: os.path.join(os.path.expanduser(
                x.get('root')),
                'config',
                'stages',
                x.get('name',
                      'default')) if isinstance(x,
                                                dict) else os.path.join(
                config_root, x),
            stages)

        return [lambda: mirror_put(local_root, '/') for local_root in
                upload_roots]

    def upload_crontab(self, config, env_option, crontab_root):
        user = config.get('user')
        crontab_file = config.get('crontab', env_option + '.conf')
        position = config.get('position')

        if not user:
            self.warn(
                "no user found,skip deploy crontab file `%s'" % config.get(
                    crontab_file))
            return

        crontab_path = os.path.expanduser(
            os.path.join(crontab_root, 'config', 'crontab', crontab_file))

        if os.path.exists(crontab_path):
            if position is None or current_machine(position):
                put(crontab_path, '/tmp/cabric_crontab')
                run('crontab < /tmp/cabric_crontab', user)
                run('rm -f /tmp/cabric_crontab')
                pass
            else:
                self.warn("crontab doesn't need"
                          " to deploy this machine:%s." % env.host_string)
                pass
            pass
        else:
            self.warn("crontab file `%s' not"
                      " found. skip to install it" % crontab_path)

        pass

    def upload_crontabs(self, crons_config, env_option, crontab_root):
        """
        because command list is execute after lambda define.

        if we add upload_crontab directly,
         the value of crontab_config will always be last item.

        :param crons_config:
        :param env_option:
        :param crontab_root:
        :return:
        """

        for crontab_config in crons_config:
            self.upload_crontab(crontab_config, env_option, crontab_root)

        pass

    def set_hostname(self):
        """
        set machine hostname
        :return:
        """
        try:
            host_index = env.hosts.index(env.host_string)

            if env.host_names[host_index]:
                run("hostnamectl set-hostname %s" % env.host_names[host_index])
                run("hostnamectl set-hostname %s --pretty" % env.host_names[
                    host_index])
                run("hostnamectl set-hostname %s --static" % env.host_names[
                    host_index])
                run("systemctl restart systemd-hostnamed")
                pass
        except IndexError:
            self.warn(
                "can't find current hostname config:%s" % env.host_string)
            pass

        pass

    def letsencrypt_server(self, domains):
        # generate key
        nginx_root = get_home('nginx')
        if nginx_root:
            run("systemctl restart nginx")  # make sure load new config
            run("certbot certonly --webroot -w {0} {1}".format(
                nginx_root,
                ' '.join(['-d %s' % v for v in domains])
            ))
            pass
        else:
            raise ValueError("no nginx found.skip config certificate...")
        pass

    def set_hosts(self, records):
        """
        ..experiment::

                use env-config to handle hosts file seems a bad idea.

                we already user upload hosts file, it's simple to use,
                user only need to care don't overwrite their hosts

                and if we want to support something like:

                    * delete hosts
                    * update ip

                we need do lot of work to support this.

        :param records:
        :return:
        """

        def set_host(ip, host):
            """
            if you want to use this feature for old file, use 4 space.

            limit:
                only support one2one relation

            :param ip:
            :param host:
            :return:
            """
            with fabric_settings(warn_only=True):
                match = '%s    %s' % (ip, host)
                if run('grep "%s" /etc/hosts' % match).failed:
                    run('echo "%s" >> /etc/hosts' % match)
                    # elif run('grep "%s" /etc/hosts' % host):
                    # sed -i -e "s//g" ???
                    #     pass
            pass

        [set_host(v[0], v[1]) for v in records]
        pass

    def set_dns_list(self, dns_list):

        for dns in dns_list:
            isp = dns.get('isp')

            if isp == '/etc/hosts':
                self.set_hosts(dns.get('records'))
            elif isp == 'dnspod.cn':
                domain = dns.get('domain')
                if domain:
                    dns_client = DNSPod()
                    dns_client.bind_domain(domain)
                    for record in dns.get('records', []):
                        dns_client.bind_record(**record)
                        pass
                    pass
            else:
                self.error("illegal dns config:%s" % dns)
                pass
            pass

        pass

    def set_ssl_config(self, ssl_config, env_name):
        """ Not support parallel workflow
        :param ssl_config:
        :param env_name:
        :return:
        """

        # verify machine which to use
        encrypt_position = ssl_config.get('encrypt-position', 0)
        try:
            use_host = env.hosts[encrypt_position]
            if use_host != env.host_string:
                return
        except IndexError:
            self.error("`ssl.encrypt-position' value is invalid.")

        # verify domains
        domains = ssl_config.get('domains')
        if not domains and not isinstance(domains, list):
            self.error("`ssl.domains' must be config.")

        # set dh_param
        dh_param = ssl_config.get('dhparam')
        if dh_param:
            dh_param_file = dh_param['path']
            dh_param_length = dh_param.get('length', 4096)

            run("test -f {0} || openssl dhparam -out {0} {1}".format(
                dh_param_file, dh_param_length))
            pass

        def create_certicifate():
            try:
                self.letsencrypt_server(domains)
            except ValueError as e:
                self.warn(e)

                # get certificate
            certificate_remote_dir = "/etc/letsencrypt/live/" + domains[0]
            fullchain = run('cat %s' % os.path.join(certificate_remote_dir,
                                                    'fullchain.pem'))
            private_key = run(
                'cat %s' % os.path.join(certificate_remote_dir, 'privkey.pem'))

            print(fullchain)
            print(private_key)
            pass

        # try to manage load balancer
        load_balancer = ssl_config.get('load-balancer')
        if load_balancer:
            lb_isp = load_balancer.get('isp')
            upstream_mode = load_balancer.get('upstream-mode')

            if lb_isp.lower() == 'qingcloud.com':
                from cabric.cloud.qingcloud import QingCloud
                client = QingCloud()
                client.connect(load_balancer['zone'])
                client.connector.debug = self.options.debug

                if upstream_mode:
                    create_certicifate()
                    return

                # try to set forward policy
                policy_name = 'letsencrypt-' + env_name
                policy = client.get_or_create_loadbalancer_policy(policy_name)

                # try to set forward rule
                rules = [{
                    'loadbalancer_policy_rule_name': domain,
                    'rule_type': 'url',
                    'val': '^/.well-known'
                } for domain in ssl_config['domains']]

                for rule in rules:
                    client.get_or_add_loadbalancer_policy_rules(
                        policy['loadbalancer_policy_id'], rule)

                client.apply_loadbalancer_policy(
                    policy['loadbalancer_policy_id'])

                http_listener = load_balancer.get('http-listener')

                # try to set backend
                # ..note::
                #   please make sure you backend works right.
                backend = load_balancer.get('backend')
                backend.update({
                    'loadbalancer_backend_name': policy[
                        'loadbalancer_policy_name'],
                    'loadbalancer_policy_id': policy['loadbalancer_policy_id']
                })
                if http_listener and backend:
                    client.get_or_add_load_balancer_backends(http_listener,
                                                             backend)
                    pass

                create_certicifate()
                pass
            elif lb_isp is None:
                self.warn("load balancer isp not specified."
                          "skip config load balancer")
                pass
            else:
                self.warn("unknown isp for load balancer %s,"
                          "skip config load balancer" % lb_isp)
                pass
            pass
        else:
            create_certicifate()
            pass
        pass

    def set_timezone(self, timezone):
        """
        should limit user input

        ..todo:
            danger if we don't limit timezone.
            something like this  `Asian/Shanghai && rm -rf /`

        timedatectl list-timezones
        timedatectl

        :param timezone:
        :return:
        """
        run('timedatectl set-timezone %s' % timezone)
        pass

    def run(self, options):
        """
        ..note::
            unstable feature.

            current we use rsync instead fabric put.
            but rsync may cause permission errors.

        :param options:
        :return:
        """

        package_root, config_root, fabric_root = get_roots(options.dir)

        bind_hosts(fabric_root, options.env, options.parallel,
                   options.hosts_file)

        # try upload repo config if it can recognize
        using_config = os.path.join(package_root, options.env)
        try:
            env_config = json.load(
                open(os.path.join(using_config, 'env.json'), 'r'))
        except ValueError:
            self.error("Invalid json syntax:%s" % os.path.join(using_config,
                                                               'env.json'))
            pass

        command_list = []

        if not options.skip_upload:
            [command_list.append(v) for v in
             self.upload_config_file(config_root,
                                     options.env,
                                     env_config.get('stages', []))]
            pass

        if not options.skip_hostname:
            command_list.append(lambda: self.set_hostname())
            pass

        timezone = env_config.get('timezone')

        if not options.skip_timezone and timezone:
            command_list.append(lambda: self.set_timezone(timezone))
            pass

        dns_list = env_config.get('dns-list')

        if not options.skip_dns and dns_list:
            command_list.append(lambda: self.set_dns_list(dns_list))
            pass

        services = env_config.get('services', [])

        if not options.skip_enable_services and services:
            command_list.append(lambda: self.enable_services(services))

        if options.reload:
            command_list.append(lambda: self.reload(options.reload, services))
            pass

        if options.restart:
            command_list.append(
                lambda: self.restart(options.restart, services))
            pass

        if options.start:
            command_list.append(
                lambda: self.start(options.start, services))
            pass

        if options.stop:
            command_list.append(
                lambda: self.stop(options.stop, services))
            pass

        crons_config = env_config.get('crons', [])
        if not options.skip_crontab and crons_config:
            command_list.append(
                lambda: self.upload_crontabs(crons_config, options.env,
                                             options.dir))
            pass

        ssl_config = env_config.get('ssl', {})

        if options.enable_certbot and ssl_config:
            command_list.append(
                lambda: self.set_ssl_config(ssl_config, options.env))
            pass

        execute(command_list)
        pass

    @classmethod
    def add_arguments(cls):
        """
        sub parser document
        """
        return [
            (('--skip-enable-services',),
             dict(action='store_true', help='skip enable system services', )),
            (('--skip-upload',),
             dict(action='store_true', help='skip upload config files', )),
            (('--skip-crontab',),
             dict(action='store_true', help='skip upload crontab', )),
            (('--skip-timezone',),
             dict(action='store_true', help='skip set timezone', )),
            (('--skip-hostname',),
             dict(action='store_true', help='skip set hostname', )),
            (('--skip-dns',),
             dict(action='store_true', help='skip config dns', )),
            (('--enable-certbot',), dict(action='store_true',
                                         help='enable config ssl certificate',
                                         )),
            (('--reload',), dict(nargs='+', help='set reload service', )),
            (('--restart',), dict(nargs='+', help='set restart service', )),
            (('--start',), dict(nargs='+', help='set start service', )),
            (('--stop',), dict(nargs='+', help='set stop service', )),
            (('--parallel', '-P'), dict(action='store_true',
                                        help='default to parallel execution'
                                             ' method', )),
        ]

    pass



# -*- coding: utf-8 -*-

import os
import py_compile
import sys

from cliez.component import Component


class CompileComponent(Component):
    # un-comment this can disable global options
    # exclude_global_option = True

    def run(self, options):
        """

        :param options:
        :return:
        """
        tag = 'o'

        for v in os.walk(options.dir):
            for base_name in v[2]:
                if not base_name.endswith('.py'):
                    continue
                file_name = os.path.join(v[0], base_name)

                kwargs = {

                }

                if sys.version > (3, 2):
                    kwargs = {
                        'optimize': 2
                    }

                    if options.keep_document:
                        kwargs = {
                            'optimize': 1
                        }
                        tag = 'c'

                    if options.original:
                        kwargs = {
                            'optimize': 1
                        }

                        tag = 'c'

                    pass

                py_compile.compile(file_name, cfile=file_name + tag, **kwargs)

                self.logger.debug('compile file %s' % file_name)
                pass
            pass
        pass

    @classmethod
    def add_arguments(cls):
        """
        sub parser document
        """
        return [
            (('--keep-document',), dict(help='use set optimize to 1', )),
            (('--original',), dict(help='use set optimize to 1', )),
        ]
        pass

    pass



# -*- coding: utf-8 -*-

import os

from cliez.component import Component


class MiniComponent(Component):
    def check(self):
        rtn = os.system("pyminifier --version > /dev/null")
        return True if rtn == 0 else False

    def minifier(self, infile):
        outfile = self.options.outdir + infile.replace(self.options.indir, '')
        outdir = os.path.dirname(outfile)
        _, infile_extension = os.path.splitext(infile)

        if self.options.dry_run:
            self.warn(outfile, prefix="[dry-run]: ", suffix="")
            return

        # currently,if outdir is file, an error caused and we don't handle it.

        if not os.path.exists(outdir):
            os.makedirs(outdir)

        if infile_extension in self.options.extension:
            cmd = "pyminifier {} > {}".format(infile, outfile)

            if self.options.prepend:
                cmd = cmd.replace('pyminifier',
                                  'pyminifier --prepend=%s' %
                                  self.options.prepend)
                pass

            if self.options.debug:
                self.logger.debug(cmd)

            os.system(cmd)
        else:
            with open(outfile, 'wb') as fw, open(infile, 'rb') as fr:
                fw.write(fr.read())
                pass
            pass
        pass

    def run(self, options):
        """
        :param options:
        :return:
        """
        if not self.check():
            self.error("please make sure you have installed pyminifier.")

        options.indir = os.path.abspath(options.indir)

        # build exclude dirs pattern
        compare_dirs = []
        for v in options.exclude_dir:
            compare_dirs.append('/' + v)
            compare_dirs.append('/' + v + '/')
            pass

        for path, subdirs, files in os.walk(options.indir):
            skip_path = [True for d in compare_dirs if d in path]
            if skip_path:
                continue

            for name in files:
                self.minifier(os.path.join(path, name))

        if self.options.with_release:
            os.chdir(self.options.outdir)
            if os.path.exists("setup.py"):
                cmd = "python setup.py sdist upload -r %s" % \
                      self.options.with_release
                if self.options.debug:
                    self.print_message(cmd)
                os.system(cmd)
                pass
            else:
                self.warn("no setup.py found,skip release")
            pass
        pass

    @classmethod
    def add_arguments(cls):
        """
        python web project deploy tool
        """
        return [
            (('indir',), dict(
                help='minifier root,default is current directory',
            )),
            (('outdir',), dict(
                help='minifier root,default is current directory',
            )),
            (('--with-release',),
             dict(help='release package to pypi server', )),
            (('--extension',),
             dict(nargs='+', default=['.py'],
                  help='file extension', )),
            (('--exclude-dir',),
             dict(nargs='+', default=['.git', 'build', '.idea', 'dist'],
                  help='directory to exclude', )),
            (('--prepend',),
             dict(help='add license to minifier file', )),
            (('--dry-run',),
             dict(action='store_true',
                  help='show file list instead filter it', )),
        ]

    pass



# -*- coding: utf-8 -*-

from cliez.component import Component


class UpdateComponent(Component):
    # un-comment this can disable global options
    # exclude_global_option = True

    def run(self, options):
        """
        :param options:
        :return:
        """

        self.print_message("plan feature")
        pass

    @classmethod
    def add_arguments(cls):
        """
        sub parser document
        """
        return [
            (('--node', '-p'),
             dict(nargs='+', help='install sub node settings', )),
        ]
        pass

    pass



# -*- coding: utf-8 -*-

import logging
import os

import requests

logger = logging.getLogger(__name__)


class DNSPod(object):
    def __init__(self, root='https://dnsapi.cn', api_format='json',
                 domain=None):
        super(DNSPod, self).__init__()
        self.logger = logger
        self.root = root
        self.format = api_format
        self.domain = domain
        pass

    def get_token(self):
        """
        currently we get dnspod token from OS environment.
        :return:
        """
        return os.environ.get("DNSPOD_LOGIN_TOKEN")

    def validated_data(self, response, excepted_code):
        """
        check dnspod api response

        :param response:
        :param excepted_code:
        :return:
        """
        try:
            rtn = response.json()

            if rtn.get('status') and rtn['status'].get('code') == "1":
                self.logger.debug("request success,json data:%s" % rtn)
                return rtn
            else:
                log_msg = (
                    "request fail.\n"
                    "\turl:\t\t\t%s\n"
                    "\trequest:\t\t%s\n"
                    "\tresponse:\t\t%s" % (response.url,
                                           response.request.body,
                                           response.text))

                if rtn['status']['code'] in excepted_code:
                    logger_handler = self.logger.warning
                else:
                    logger_handler = self.logger.error
                    rtn = None
                    pass

                logger_handler(log_msg)
                return rtn

        except Exception as e:
            self.logger.error(
                "parse json fail, url:%s, rtn:%s, exception:%s" % (
                    response.url, response.text, e))
            return
        pass

    def request(self, api_path, data={}, excepted_code=[]):
        """
        generic request method
        :param api:
        :param data:
        :return:
        """

        api = self.root + api_path

        token = self.get_token()
        if not token:
            raise ValueError("please set `DNSPOD_LOGIN_TOKEN' environment")

        data.update({
            "login_token": token,
            "format": "json"
        })

        response = requests.post(api, data)
        return self.validated_data(response, excepted_code)

    def get_version(self):
        """
        get dnspod api version
        :return: version or raise ValueError
        """

        rtn = self.request('/Info.Version')
        if not rtn:
            raise ValueError

        return rtn["status"]["message"]

    def bind_domain(self, domain=None):
        """
        if domain is exist,we consider it is works.

        if user set domain, after bind progress, self.domain will set to domain

        :param domain:
        :return:
        """

        domain = domain or self.domain

        if not domain:
            raise ValueError("please set domain or self.domain value")

        rtn = self.request('/Domain.Create', {
            "domain": domain
        }, ["7"])

        if rtn['status']['code'] in ["1", "7"]:
            self.domain = domain
            return True
        else:
            return False
        pass

    def get_record(self, query, length=32):
        """
        get domain record.
        :param data:
        :param length: currently,only support max 32 sub domain
        :return: record, if not exist return None
        """
        if not self.domain:
            raise ValueError("please set `self.domain' or"
                             " use `bind_domain' to rebind domain")

        rtn = self.request('/Record.List', {
            "domain": self.domain,
            "sub_domain": query['sub_domain'],
            "length": length,
        })

        if rtn:
            for record in rtn['records']:

                compared = [
                    True if record['value'] == query['value'] else False,
                    True if record['line'].encode('utf-8') == query[
                        'record_line'] else False,
                    True if record['type'] == query['record_type'] else False
                ]

                if False not in compared:
                    return record

                pass
        pass

    def get_record_id(self, query, length=32):
        """
        get domain record id named by dnspod
        :param query:
        :param length:
        :return: record id.if not exist,return None
        """

        record = self.get_record(query, length)

        if record:
            return record['id']
        pass

    def remove_domain(self):
        """

        :param api_data:
        :return:
        """

        if not self.domain:
            raise ValueError(
                "please set `self.domain' or use `bind_domain' to"
                " rebind domain")

        return self.request('/Domain.Remove', {
            "domain": self.domain,
        })
        pass

    def remove_record(self, api_data):
        """
        only exist record will be deleted
        :param api_data:
        :return:
        """

        if not self.domain:
            raise ValueError(
                "please set `self.domain' or use `bind_domain' to"
                " rebind domain")

        record_id = self.get_record_id(api_data)

        if record_id:
            return self.request('/Record.Remove', {
                "domain": self.domain,
                "record_id": record_id
            })
        pass

    def modify_record(self, record_id, api_data):
        """
        modify record

        :param record_id: dnspod record id
        :param api_data:
        :return:
        """

        if not self.domain:
            raise ValueError(
                "please set `self.domain' or use `bind_domain' to"
                " rebind domain")

        api_data.update({
            "domain": self.domain,
            "record_id": record_id,
        })

        return self.request('/Record.Modify', api_data)

    def bind_record(self, value, record_type, sub_domain='@', record_line='默认',
                    weight=None, mx_order=5, ttl=600, active=True,
                    overwrite=True, **kwargs):
        """
        :param value:
        :param record_type:
        :param sub_domain:
        :param record_line:
        :param weight: your account must be VIP account
        :param mx_order:
        :param ttl:
        :param active:
        :param overwrite: rewrite if record is exist. default is True
        :param kwargs:
        :return: dict or None if fail
        """

        if not self.domain:
            raise ValueError("please set `self.domain' or use `bind_domain' to"
                             " rebind domain")

        data = {
            "domain": self.domain,
            "record_type": record_type.upper(),
            "sub_domain": sub_domain,
            "record_line": record_line,
            "value": value,
            "ttl": ttl,
            "status": "enable" if active else "disable",
        }

        if data['record_type'] == 'MX':
            data['mx'] = mx_order
            pass

        if weight:
            data['weight'] = weight
            pass

        rtn = self.request('/Record.Create', data, excepted_code=['104'])

        if rtn:
            if rtn['status']['code'] == '104':
                if overwrite is True:  # domain exists
                    record_id = self.get_record_id(data)
                    self.modify_record(record_id, data)
                    pass
                pass
            return rtn
        pass

    pass



# -*- coding: utf-8 -*-

import json
import os
from getpass import getpass

import requests
from Crypto.PublicKey import RSA
from cliez.component import Component
from fabric.context_managers import settings
from fabric.operations import put as fabric_put
from git import config as pygit

from cabric.utils import get_roots, bind_hosts, \
    execute, get_repo, put, run, \
    get_home, get_git_host, known_host, cd

try:
    input = raw_input
except NameError:
    pass


class DeployComponent(Component):
    def create_github_key(self, private_key, github, remote_user,
                          key_length=8192):

        # login
        def use_personal_token():
            """
            ..note::
                token was limited when user use strict permission
            :return:
            """
            git_config_path = os.path.expanduser('~/.gitconfig')
            if os.path.exists(git_config_path):
                git_config = pygit.GitConfigParser(git_config_path)
                git_config.read()
                access_token = git_config.get_value('github', 'token',
                                                    default='')
                username = git_config.get_value('github', 'user', default='')
                pass

                if access_token and username:
                    return (username, access_token)
            pass

        def use_github_password():
            username = input('Github Account:')
            password = getpass('Github Password:')
            auth_config = (username, password)
            return auth_config

        authorized_methods = [
            use_personal_token,
            use_github_password
        ]

        auth_config = None

        for v in authorized_methods:
            temp_config = v()
            response = requests.get(
                'https://api.github.com/repos/{}/keys'.format(github),
                auth=temp_config)
            if response.status_code != 200:
                self.warn("use `%s' method login to github is failed,"
                          "try next way." % v.__name__.replace('use_',
                                                               '').replace('_',
                                                                           '-')
                          )
                if self.options.debug:
                    self.warn("github-api"
                              " request: %s" % 'https://api.github.com'
                                               '/repos/{}/keys'.format(github))
                    self.warn(
                        "github-api status-code: %d" % response.status_code)
                    self.warn("github-api auth: %s" % ' '.join(temp_config))
                    self.warn("github-api return: %s" % response.text)
                    pass
                continue

            auth_config = temp_config
            keys = response.json
            key_title = remote_user + '@cabric'
            break

        if not auth_config:
            self.error("login github failed or project not exists.")

        try:
            uploaded_keys = [v for v in keys if v.get('title') == key_title]
        except TypeError:
            uploaded_keys = []

        # clean old key if exists
        for v in uploaded_keys:
            response = requests.delete('https://api.github.com'
                                       '/repos/{}/keys/{}'.format(github,
                                                                  v.get('id')),
                                       auth=auth_config)
            if response.status_code != 204:
                self.error("delete old deploy key failed,"
                           "please try it later."
                           "server response:\n%s" % response.text)
                pass

        # generate new key
        self.print_message(
            "Generating deploy key... this will cost about 1 minute...")
        key = RSA.generate(key_length)

        private_key_dir = os.path.dirname(private_key)
        if not os.path.exists(private_key_dir):
            os.makedirs(private_key_dir, 0700)
            pass

        with open(private_key, mode='w') as fp:
            fp.write(key.exportKey('PEM'))
            pass

        pubkey = key.publickey()
        pubkey_path = os.path.splitext(private_key)[0] + '.pub'

        with open(pubkey_path, mode='w') as fp:
            fp.write(key.exportKey('OpenSSH'))
            pass

        response = requests.post(
            'https://api.github.com/repos/{}/keys'.format(github),
            data=json.dumps({
                'title': key_title,
                'key': pubkey.exportKey('OpenSSH'),
                'read_only': True,
            }), auth=auth_config)

        if response.status_code != 201:
            self.error(
                "create key failed.server response:\n%s" % response.text)
            pass

        pass

    def upload_deploy_key(self, private_key, remote_user, project_name):
        """
        upload deploy key

        :param string private_key: private key local path
            default is ~/.ssh/.deploies/`github`.rsa
        :param string remote_user: remote user name to deploy
        :param string project_name: a project name
        :param string github: github repo name
        :param bool force_renew: try to replace deploy
                    key when use auto-generate
        :param int key_length: must a legal ssh key length value.
                    default is 8192


        ..note::

            if you use github and want to
             use auto generate private-key feature.

            there is two ways can do this:

                - you must set access token in your ~/.gitconfig file
                - you must disable github two-factor authentication,
                    input your username and password.

            currently, we use `<remote_user>@cabric` as our deploy key name.
            so if you upload your key use to other purpose,
             don't use `@cabric` as key suffix.


            if github deploy key already exist and
            you want to replace deploy key. you must set `--fresh-new' option.


            cabric allow each machine deploy multiple github project,
            but disallow deploy same name project in one user.

            if you still want do this.
                - you can set github value if you use it.
                - deploy them in different remote user.


        ..note::

            currently, this only works on linux.

        :return:
        """

        if not os.path.exists(private_key):
            self.error(
                "deploy key `%s' is not exists,please set it." % private_key)

        if os.path.exists(private_key):
            self.print_message("upload deploy key...")
            remote_key = self.get_remote_key(remote_user, project_name)
            remote_key_root = os.path.dirname(remote_key)

            run('test -e {0} || mkdir -p {0}'.format(remote_key_root),
                remote_user)
            with settings(warn_only=True):
                run('chmod 700 -Rf {}'.format(remote_key_root), remote_user)

            fabric_put(private_key, remote_key)
            run('chmod 600 -f {}'.format(remote_key))
            run('chown {1} -f {0}'.format(remote_key, remote_user))
            pass

        pass

    def get_project_python(self, user, project_name):
        project_path = self.get_remote_project_path(user, project_name)
        with settings(warn_only=True):
            version = run("test -f {0}/.python-version &&"
                          " cat {0}/.python-version".format(project_path))
            version = version.strip("\n")
            return version
        pass

    def install_project_python(self, user, project_name):
        remote_version = self.get_project_python(user, project_name)
        if remote_version:
            run('pyenv install -s %s' % remote_version)
        pass

    def install_requirements(self, user, project_name, pip='pip'):
        """
        when requirements file exits. install it.

        :param user: remote user to deploy
        :param project_name: project name
        :return:
        """

        project_path = self.get_remote_project_path(user, project_name)
        python_version = self.get_project_python(user, project_name)

        requirement_files = [
            os.path.join(project_path, 'requirements.txt'),
            os.path.join(project_path, 'requirements', 'test.txt'),
            os.path.join(project_path, 'requirements', 'private.txt'),
        ]

        for f in requirement_files:
            # 有版本bug
            # " install -U  --upgrade-strategy only-if-needed -r {0} || "

            run("test -f {0} && /usr/local/var/pyenv/versions/{1}/bin/pip"
                " install -U  -r {0} || "
                "echo '{0} not exist,skip install...'".format(f,
                                                              python_version))
            pass

        requirement_files = [
            os.path.join(project_path, 'requirements-static.txt'),
            os.path.join(project_path, 'requirements', 'zip.txt'),
            os.path.join(project_path, 'requirements', 'private-static.txt'),
            os.path.join(project_path, 'requirements', 'test-static.txt'),
        ]

        for f in requirement_files:
            run("test -f {0} && /usr/local/var/pyenv/versions/{1}/bin/pip"
                " install -r {0} || "
                "echo '{0} not exist,skip install...'".format(f,
                                                              python_version))
            pass

        pass

    def migrate_db(self, user, project_name, django_settings='web.online'):
        """
        try migrate database

        :param user: remote user
        :param project_name: project name
        :return:
        """
        project_path = self.get_remote_project_path(user, project_name)

        with cd(project_path):
            run(
                'test -f %(project_path)s/manage.py && '
                'python %(project_path)s/manage.py migrate '
                '--settings=%(settings)s'
                ' || echo "skip migrate database"' % {
                    'project_path': project_path,
                    'settings': 'web.online'
                },
                user=user
            )
            pass

        pass

    def compile_templates(self, user, project_name):
        """
        try compile pug template files

        ..note::

            cabric use project root as pug basedir root.
            if you don't use this path,
            use `--skip-compile-templates` to skip this progress.


        :param user: remote user
        :param project_name: project name
        :return:
        """
        project_path = self.get_remote_project_path(user, project_name)
        run('which pug && pug -E html -b {0} {0} '
            '|| echo "skip parser jade file"'.format(project_path))
        pass

    def upload_resources(self, user, project_name, working_root=None,
                         static_prefix=None):
        """
        upload static resoures file is exists.

        only works for django+nginx(start with nginx user) project.

        cabric will upload:

            - static    django resource directory
            - assets    webpack resource directory

        ..todo::
            use remote path to validate install

        :return:
        """

        remote_root = self.get_remote_project_path(user, project_name)

        working_root = working_root or os.getcwd()
        django_manage = os.path.join(working_root, 'manage.py')

        if not os.path.exists(django_manage):
            self.warn(
                "local root is not a django project,skip upload resources")
            return

        with settings(warn_only=True):
            if run("test -f %s/manage.py" % remote_root).failed:
                self.warn("deploy project is not django project,"
                          "skip upload resources")
                return
            pass

        try:
            nginx_home = get_home('nginx')
        except ValueError:
            self.warn("remote server only support nginx "
                      "and must use nginx user start,"
                      "skip deploy static resources...")
            return

        static_prefix = static_prefix or ''
        nginx_static_root = os.path.join(nginx_home, static_prefix, 'static')

        # collect static files by user
        # fabric_local('python manage.py collectstatic --noinput')

        with settings(warn_only=True):
            run('test -e {0} || mkdir -p {0}'.format(nginx_static_root))

        static_root_list = [
            os.path.join(working_root, 'static'),
            os.path.join(working_root, 'assets')
        ]

        for v in static_root_list:
            if os.path.exists(v):
                put(v, nginx_static_root)
                pass
            pass

        pass

    def upload_javascripts(self, remote_user, project_name,
                           working_root=None):

        remote_path = self.get_remote_project_path(remote_user, project_name)
        working_root = working_root or os.getcwd()

        webpack_stats_file = os.path.join(working_root, 'javascripts',
                                          'webpack-stats.json')
        remote_javascripts_dir = os.path.join(remote_path, 'javascripts')

        with settings(warn_only=True):
            if os.path.exists(webpack_stats_file):
                if not run("test -d %s" % remote_javascripts_dir).failed:
                    put(webpack_stats_file, remote_javascripts_dir)
                pass

        pass

    def upgrade(self, remote_user, project_name,
                repo, branch, commit=None):
        """
        upgrade source code

        :param remote_user:deploy username
        :param project_name: project name
        :param repo:git repo address
        :param branch:which branch to deploy

        ..note::
            currently,if remote machine already cloned from repo,
            branch can't be change.

            if you really need to change branch.
            you have to remove remote project directory,
            do upgrade again.

        :param commit:which commit to deploy,default use latest commit,
            support tags

        ..note::
            commit or tag must be valid in branch


        :return:
        """

        host = get_git_host(repo)
        known_host(host, remote_user)

        remote_path = self.get_remote_project_path(remote_user, project_name)
        deploy_key = self.get_remote_key(remote_user, project_name)

        with settings(warn_only=True):
            if run("test -d %s/.git" % remote_path).failed:
                parent_path = os.path.dirname(remote_path)
                run('test -d {0} || mkdir {0}'.format(parent_path),
                    remote_user)
                with cd(parent_path):
                    run('git clone {} -b {} {}'.format(repo, branch,
                                                       remote_path),
                        remote_user)
                    run("cd {} && git config core.fileMode false".format(
                        remote_path), remote_user)

        run("cd {} && git pull origin {}".format(remote_path, branch),
            remote_user)
        run("cd {} && git pull origin {} --tags".format(remote_path, branch),
            remote_user)

        if commit:
            # make sure there is no merge commit on remote server
            # run("cd {} && git checkout -- .".format(remote_path),
            #     remote_user)
            run("cd {} && git checkout {}".format(remote_path, commit),
                remote_user)

        pass

    def get_remote_key(self, user, project_name):
        """
        ..note::
            currently,we force use ~/.ssh/id_rsa
            if you saved more than on project deploy in same remote_user.
            use `--with-deploy-key' to replace remote key.


        ..todo::
            make sure only one project is deploying


        :param user:
        :param project_name:
        :return:
        """

        # return '{}/.cabric/{}.rsa'.format(get_home(user), project_name)
        return '%s/.ssh/id_rsa' % get_home(user)

    def get_remote_project_path(self, user, project_name):
        """
        ..note::
            it must be called in lambda

        :param user:
        :param project_name:
        :return:
        """

        return os.path.join(get_home(user), project_name)

    def run(self, options):
        """ workflow
        * basic check

            - user value must be set
            - currently, repo only support git
            - for safety reason,we don't allow private_key path
              occur in project path

        * upload deploy-key when not exist.
        * git clone with project-deploy-key.
        * install requirements.txt package if exist
        * install nodejs local-package if exist
        * parse template file
        * git pull or checkout commit
        * add crontab if user set
        * finish

        :param options:
        :return:
        """

        if options.verbose == 3:
            options.debug = True

        package_root, _, fabric_root = get_roots(options.dir)
        bind_hosts(fabric_root, options.env, options.parallel,
                   options.hosts_file)

        using_config = os.path.join(package_root, options.env)
        project_root = os.path.dirname(package_root)

        try:
            packages_config = json.load(
                open(os.path.join(using_config, 'env.json'), 'r'))
        except ValueError:
            self.error("Invalid json syntax:%s" % os.path.join(using_config,
                                                               'env.json'))

        config = packages_config.get('deploy', {})

        # basic check
        if not config.get('user'):
            self.error("user must be set")

        user = config['user']
        repo = config.get('repo', get_repo())
        branch = config.get('branch', 'master')

        if repo.find('git') != 0:
            self.error("sorry,currently we only support git+ssh mode")

        github = config.get('github')

        project_name = github if github else os.path.basename(repo).replace(
            '.git', '')
        private_key = os.path.expanduser(config.get('private_key',
                                                    '~/.ssh/.deploies/'
                                                    '%s.rsa' % project_name))

        if project_root in private_key:
            self.error("for safety reason,"
                       "we don't allow private_key path in project's path")

        command_list = []

        if options.with_deploy_key:
            if options.force_renew or (
                    not os.path.exists(private_key) and github):
                self.create_github_key(private_key, github, user)

            command_list.append(
                lambda: self.upload_deploy_key(private_key, user,
                                               project_name))

        if not options.skip_source_code:
            command_list.append(
                lambda: self.upgrade(user, project_name, repo, branch,
                                     commit=options.commit))

        if not options.skip_requirements:
            command_list.append(
                lambda: self.install_project_python(user, project_name))
            command_list.append(
                lambda: self.install_requirements(user, project_name))

        if options.enable_compile_templates:
            command_list.append(
                lambda: self.compile_templates(user, project_name))

        if not options.skip_migrate:
            command_list.append(
                lambda: self.migrate_db(user, project_name,
                                        config.get('settings', 'web.online')))

        if not options.skip_upload_resources:
            resource_dir = os.path.expanduser(options.resource_dir)

            command_list.append(
                lambda: self.upload_resources(user, project_name,
                                              working_root=resource_dir,
                                              static_prefix=config.get(
                                                  'static-prefix')))
            command_list.append(
                lambda: self.upload_javascripts(user, project_name,
                                                working_root=resource_dir,
                                                ))

        execute(command_list)
        pass

    @classmethod
    def add_arguments(cls):
        """
        python web project deploy tool
        """
        return [
            (('commit',), dict(nargs='?',
                               help='set which commit to deploy,'
                                    'default is latest version', )),
            (('--parallel', '-P'), dict(action='store_true',
                                        help='default to parallel '
                                             'execution method', )),
            (('--with-deploy-key',),
             dict(action='store_true', help='upload deploy key', )),
            (('--force-renew',), dict(action='store_true',
                                      help='only works when user '
                                           'set github value', )),
            (('--skip-source-code',),
             dict(action='store_true', help='skip upgrade source code', )),
            (('--skip-requirements',),
             dict(action='store_true', help='skip install requirements', )),
            (('--skip-migrate',),
             dict(action='store_true', help='skip migrate', )),
            (('--skip-upload-resources',),
             dict(action='store_true', help='skip upload resources', )),
            (('--enable-compile-templates',),
             dict(action='store_true', help='enable compile templates', )),
            (('--resource-dir',),
             dict(nargs='?', default=os.getcwd(),
                  help='set resource upload directory')),
        ]
        pass

    pass



# -*- coding: utf-8 -*-

import json
import os
import subprocess

from cliez.component import Component

from cabric.utils import get_roots

try:
    from shlex import quote as shell_quote
except ImportError:
    from pipes import quote as shell_quote


class HardlineComponent(Component):
    def run(self, options):
        """
        :param options:
        :return:
        """

        package_root, _, _ = get_roots(options.dir)
        config_file = os.path.join(package_root, 'hardlines.json')

        if not os.path.exists(config_file):
            raise OSError("%s not exists" % config_file)

        try:
            hardlines = json.load(open(config_file, 'r'))
        except ValueError:
            self.error("Invalid json syntax:%s" % config_file)
            return False

        # show keys and exit
        if not options.node:
            docs = [(v, hardlines[v].get('title', '')) for v in
                    hardlines.keys()]

            self.print_message("All Hardline:\n")
            self.print_message(
                "\n".join([v[0] + "\t" + v[1] for v in docs]))

            return

        node = hardlines.get(options.node, {})

        if not node:
            self.error("Invalid node :%s" % options.node)
            return False

        workflow = [
            v for v in node.get('workflow', [])
            if v[0:3] in ('cab', 'fab') and '&' not in v and '|' not in v
            and '>' not in v and '<' not in v
            ]

        if not workflow:
            self.error("No workflow set.")
            return False

        commands = "\n".join(workflow)

        if options.preview:
            self.print_message("WorkFlow:\n")
            self.print_message(commands)
            return

        commands = "shopt -s expand_aliases\n" \
                   "source ~/.bash_profile\n" + commands

        if options.debug:
            self.print_message("commands will be execute:\n" + commands)
            pass

        subprocess.call(commands, shell=True, executable='/bin/bash')
        pass

    @classmethod
    def add_arguments(cls):
        """
        sub parser document
        """
        return [
            (('--preview',),
             dict(action='store_true', help='show task', )),
            (('node',),
             dict(nargs='?', help='which node to active', )),
        ]
        pass

    pass



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import logging
import unittest

from cabric.cloud.qingcloud import QingCloud

logging.basicConfig(level=logging.DEBUG)


class QingCloudTestCase(unittest.TestCase):
    """
    please set os.environment. value
    """

    def setUp(self):
        logging.basicConfig(level=logging.DEBUG)
        self.client = QingCloud()
        self.drop_config = []
        pass

    def tearDown(self):
        pass

    def test_init(self):
        self.assertEqual(None, self.client.connector)
        pass

    def test_connector(self):
        """
        Don't forget set os.env['QINGCLOUD_ACCESS_KEY']
         and os.env['QINGCLOUD_ACCESS_SECRET']
        :return:
        """
        self.client.connect('pek-2')
        self.assertNotEqual(None, self.client.connector)
        pass

    def test_api(self):
        """
        Don't forget set os.env['QINGCLOUD_ACCESS_KEY'] and
         os.env['QINGCLOUD_ACCESS_SECRET']
        :return:
        """

        self.client.connect('pek2')
        result = self.client.describe_instances()
        self.assertEqual(0, result.get('ret_code', -1))
        pass

    pass



# -*- coding: utf-8 -*-

import json
import os

from cliez.component import Component
from fabric.context_managers import settings, cd

from cabric.utils import get_roots, mirror_put, run, \
    bind_hosts, execute, get_platform, run_block, \
    exist_group, exist_user

try:
    from shlex import quote as shell_quote
except ImportError:
    from pipes import quote as shell_quote


class InstallComponent(Component):
    # un-comment this can disable global options
    # exclude_global_option = True

    system_python_version = '2.7.11'

    def before_install(self, root):
        run('yum install -y rsync')
        mirror_put(root, '/etc/hosts', validate=False)
        mirror_put(root, '/root/.bash_profile', validate=False)
        pass

    def install_user(self, users):
        """
        install user

        ..note::

            user will only install on linux machine.

        :return:
        """

        def on_centos():
            """
            workflow

            * parse config and set default value
                * force set shell to /sbin/nologin when system flag is set
                * cause error when config.name not exists
            * skip create when user exists
            * create group explicitly
                when user specify and not same config.name

            :return:
            """
            for user in users:
                username = user.get('name')
                groupname = user.get('group', username)
                home = user.get('home', '/home/{}'.format(username))
                shell = user.get('shell', '`which bash`')
                system_flag = user.get('system', False)
                perm = user.get('perm')

                if system_flag:
                    shell = '/sbin/nologin'

                if not username:
                    self.error("invalid user config")

                if exist_user(username):
                    if perm:
                        run('chmod {1} {0}'.format(home, perm))
                    self.warn("user `%s' exist.skip to create." % username)
                    continue

                system_str = '-r' if system_flag else ''

                if groupname == username:
                    group_str = '-U'
                else:
                    if not exist_group(groupname):
                        run('groupadd {1} {0}'.format(groupname, system_str))
                        pass
                    group_str = '-g {}'.format(groupname)

                run('useradd {1} -d {2} {4} -s {3} {0}'.format(username,
                                                               group_str,
                                                               home, shell,
                                                               system_str))

                # todo:
                # .. if perm cause error, user will not delete
                if perm:
                    run('chmod {1} {0}'.format(home, perm))

                pass

        run_block(centos=on_centos)
        pass

    def install_package(self, root, pkgs_config):
        """
        install system package
        workflow:
            - try add gpg key
            - yum install
            - yum localinstall

        :param root: pkgs_config root directory path
        :param pkgs_config: package.json pkgs_config
        :return:
        """

        def on_centos():
            rpm_keys = pkgs_config.get('rpm-keys', [])
            if rpm_keys:
                for v in rpm_keys:
                    run("rpm --import %s" % v)
                    pass
                pass

            pkg = pkgs_config.get('yum')
            if pkg:
                mirror_put(root, '/etc/yum.repos.d', validate=False)
                run('yum install -y epel-release')
                run('yum install -y %s' % ' '.join(pkg))

            pkg_local = pkgs_config.get('yum-local', [])
            for p in pkg_local:
                with cd("/tmp"):
                    run("curl -L -O %s" % p)
                    run('yum localinstall -y %s' % os.path.basename(p))
                    pass
                pass
            pass

        def on_mac():
            pkg = pkgs_config.get('brew')
            if not pkg:
                return

            with settings(warn_only=True):
                run('brew install %s' % ' '.join(pkg))

            pass

        run_block(centos=on_centos, mac=on_mac)
        pass

    def install_pyenv(self, versions=None, skip_pkg=False):
        """
        We will install pyenv by default. and with normal use lib

        Because pyenv is awesome!!!

        :param root:
        :param skip_pkg: skip install depends package, default is False

        :return:
        """
        remote_os = get_platform()

        if remote_os == 'centos':
            if not skip_pkg:
                run('yum install -y git')
                run("yum install -y gcc gcc-c++ make autoconf certbot"
                    " libffi-devel ncurses-devel expat-devel"
                    " zlib-devel zlib libzip-devel"
                    " bzip2 bzip2-devel bzip2-libs"
                    " mariadb-devel mariadb-libs"
                    " sqlite-devel"
                    " libxml2 libxml2-devel libxslt libxslt-devel"
                    " libcurl-devel"
                    " pcre-devel pcre"
                    " libmcrypt libmcrypt-devel openssl-devel openssl-libs"
                    " libjpeg libjpeg-devel libpng libpng-devel"
                    " freetype freetype-devel "
                    " libtiff-devel lcms2-devel libwebp-devel"
                    " tcl-devel tk-devel"
                    )
            run('export PYENV_ROOT=/usr/local/var/pyenv && '
                'curl -L http://nextoa.com/'
                'bin/pyenv-install.sh  | bash')
            run('ln -sfv /usr/local/var/pyenv/bin/pyenv /usr/local/bin/pyenv')

            pass
        elif remote_os == 'mac':
            with settings(warn_only=True):
                run('brew install git pyenv')
            pass

        run('test -d /usr/local/var/pyenv/plugins/pyenv-virtualenv ||'
            ' git clone https://github.com/yyuu/pyenv-virtualenv.git'
            ' /usr/local/var/pyenv/plugins/pyenv-virtualenv')

        run('export PYENV_ROOT="/usr/local/var/pyenv/" &&'
            ' eval "$(pyenv init -)"')

        if isinstance(versions, list):
            for v in versions:
                run('export PYENV_ROOT="/usr/local/var/pyenv/" &&'
                    ' pyenv install -s %s' % v)
        elif type(versions).__name__ in ['str', 'unicode']:
            run('export PYENV_ROOT="/usr/local/var/pyenv/" &&'
                ' pyenv install -s %s' % versions)
        else:
            run('export PYENV_ROOT="/usr/local/var/pyenv/" && '
                'pyenv install -s %s' % self.system_python_version)

        command_list = [
            """grep "PYENV_ROOT" /etc/profile || \
            echo 'export PYENV_ROOT="/usr/local/var/pyenv/"' \
            >> /etc/profile""",
            """grep "pyenv init" /etc/profile || \
            echo 'eval "$(pyenv init -)"' >> \
            /etc/profile""",
            """grep "pyenv virtualenv" /etc/profile || \
            echo 'eval "$(pyenv virtualenv-init -)"' \
            >> /etc/profile""",
        ]

        if remote_os == 'mac':
            command_list = ["sudo sh -c '%s'" % shell_quote(v) for v in
                            command_list]
            pass

        for cmd in command_list:
            run(cmd)
            pass

        pass

    def install_pypi(self, pypi_configs, skip_pkg=False):
        """
        :param pip_configs:
        :param skip_pkg: skip install depends package, default is False
        :return:
        """

        def install(pypi_config):

            python_version = pypi_config.get('version')

            if python_version:
                self.install_pyenv(python_version, skip_pkg)

            used_version = python_version \
                if python_version else self.system_python_version

            for pkg in pypi_config.get('packages'):
                run('export PYENV_ROOT="/usr/local/var/pyenv/" && '
                    'export PYENV_VERSION=%s && '
                    'pip install %s' % (used_version, pkg))
                pass

            pass

        [install(pypi_config) for pypi_config in pypi_configs]
        pass

    def install_node(self, root, pkgs_config, skip_pkg=False):
        """
        global node package denends

        ..note::

            this will install global package.

        :param root: pkgs_config root directory path
        :param pkgs_config: package.json pkgs_config
        :param skip_pkg: skip install depends package, default is False
        :return:
        """

        pkg = pkgs_config.get('node')
        if not pkg:
            return

        def on_centos():
            if not skip_pkg:
                run('yum install -y epel-release')
                run('yum install -y nodejs npm')
            pass

        def on_mac():
            if skip_pkg:
                run('brew install node')
            pass

        def on_all():
            """
            plan feature
            support add extra repo source
            :return:
            """
            # npm_bin = run('which npm')
            # npm_root = npm_bin.rsplit('/', 2)[0]
            # npm_etc = os.path.join(npm_root, 'etc')
            # print(npm_root)
            run('npm install -g %s' % ' '.join(pkg))
            pass

        run_block(centos=on_centos, mac=on_mac, all=on_all)
        pass

    def run(self, options):
        """
        workflow

            * try upload repo config if it can recognize
            * execute install
            * create user if not exists (only on linux machine)
            * execute custom script
            * enable register service
            * finish

        :param options:
        :return:
        """

        package_root, config_root, fabric_root = get_roots(options.dir)
        bind_hosts(fabric_root, options.env, options.parallel,
                   options.hosts_file)

        # try upload repo config if it can recognize
        using_config = os.path.join(config_root, options.env)

        try:
            packages_config = json.load(
                open(os.path.join(package_root, options.env, 'packages.json'),
                     'r'))
        except ValueError:
            self.error("Invalid json"
                       " syntax:%s" % os.path.join(package_root,
                                                   options.env,
                                                   'packages.json'))

        try:
            env_config = json.load(
                open(os.path.join(package_root, options.env, 'env.json'), 'r'))
        except ValueError:
            self.error("Invalid json"
                       " syntax:%s" % os.path.join(package_root,
                                                   options.env,
                                                   'env.json'))

        execute_list = []

        execute_list.append(lambda: self.before_install(using_config))

        if not options.skip_pkg:
            execute_list.append(
                lambda: self.install_package(using_config, packages_config))

        if not options.skip_pyenv:
            execute_list.append(
                lambda: self.install_pyenv(packages_config.get('pyenv'),
                                           options.skip_pkg))

        if not options.skip_pypi:
            execute_list.append(
                lambda: self.install_pypi(packages_config.get('pypi', []),
                                          options.skip_pkg))

        if options.enable_node:
            execute_list.append(
                lambda: self.install_node(using_config, packages_config,
                                          options.skip_pkg))

        if not options.skip_user:
            execute_list.append(
                lambda: self.install_user(env_config.get('users', [])))

        if execute_list:
            execute(execute_list)
        else:
            self.print_message("nothing to do.")

        pass

    @classmethod
    def add_arguments(cls):
        """
        sub parser document
        """
        return [
            (('--skip-pkg',),
             dict(action='store_true', help='skip install pkg', )),
            (('--skip-pyenv',),
             dict(action='store_true', help='skip install pyenv', )),
            (('--skip-user',),
             dict(action='store_true', help='skip install user', )),
            (('--skip-pypi',),
             dict(action='store_true', help='skip install pip package', )),
            (('--enable-node',),
             dict(action='store_true', help='enable install node', )),
            (('--parallel', '-P'), dict(action='store_true',
                                        help='default to parallel execution'
                                             ' method', )),
        ]
        pass

    pass



# -*- coding: utf-8 -*-

import logging
import unittest

from cabric.dns.dnspod import DNSPod


class DnspodTestCase(unittest.TestCase):
    """
    please set os.environment.DNSPOD_LOGIN_TOKEN value
    """

    def setUp(self):
        logging.basicConfig(level=logging.DEBUG)
        self.client = DNSPod()
        self.drop_config = []
        pass

    def tearDown(self):
        if self.drop_config:
            for c in self.drop_config:
                self.client.remove_record(c)
                pass

        if self.client.domain:
            self.assertNotEqual(None, self.client.remove_domain())
        pass

    def test_api_version(self):
        self.assertIsInstance(float(self.client.get_version()), float)
        pass

    def test_bind_domain(self):
        self.assertEqual(True, self.client.bind_domain('kay8.com'))
        pass

    def test_bind_domain02(self):
        self.client = DNSPod(domain='kay8.com')
        self.assertEqual(True, self.client.bind_domain())
        pass

    def test_bind_record(self):
        self.test_bind_domain()

        self.drop_config = [
            dict(value='1.1.1.1', record_type='A', sub_domain='unittest-01'),
            dict(value='1.1.1.2', record_type='A', sub_domain='unittest-01',
                 active=False)
        ]

        [self.assertNotEqual(None, self.client.bind_record(v)) for v in
         self.drop_config]
        pass

    def test_bind_record02(self):
        self.test_bind_domain()

        self.drop_config = [
            dict(value='1.1.1.1', record_type='A', sub_domain='unittest-02',
                 record_line='电信'),
            dict(value='1.1.1.2', record_type='A', sub_domain='unittest-02',
                 record_line='移动')
        ]

        [self.assertNotEqual(None, self.client.bind_record(v)) for v in
         self.drop_config]
        pass

    def test_bind_record03(self):
        self.test_bind_domain()

        self.drop_config = [
            dict(value='1.1.1.1', record_type='A', sub_domain='unittest-01'),
            dict(value='1.1.1.2', record_type='A', sub_domain='unittest-01',
                 active=False)
        ]

        [self.assertNotEqual("1", self.client.bind_record(v)['status']['code'])
         for v in self.drop_config]
        [self.assertNotEqual("104",
                             self.client.bind_record(v)['status']['code']) for
         v in self.drop_config]
        pass

    def test_get_record_id(self, test_data=None):
        test_data = test_data or {
            "value": "1.1.1.2",
            "sub_domain": "unittest-02",
            "record_type": "A",
            "record_line": "电信",
        }

        self.test_bind_record02()
        record_id = self.client.get_record_id(test_data)
        self.assertNotEqual(None, record_id)

        return record_id

    def test_modify(self):
        test_data = {
            "value": "2.2.2.2",
            "sub_domain": "unittest-02",
            "record_type": "A",
            "record_line": "电信",
            "status": "disable",
        }

        record_id = self.test_get_record_id()
        self.assertNotEqual(None,
                            self.client.modify_record(record_id, test_data))
        pass

    pass



# -*- coding: utf-8 -*-

import os

from cliez.component import Component


class CleanComponent(Component):
    # un-comment this can disable global options
    # exclude_global_option = True

    def run(self, options):
        for v in os.walk(options.dir):
            for base_name in v[2]:
                if base_name.endswith('.pyc') or base_name.endswith('.pyo'):
                    file_name = os.path.join(v[0], base_name)
                    os.unlink(file_name)
                    self.logger.debug("delete file:%s" % file_name)
                    pass
                pass

            dir_name = os.path.basename(v[0])

            if dir_name == '__pycache__':
                try:
                    os.rmdir(v[0])
                    self.logger.debug("delete directory:%s" % v[0])
                except OSError:
                    self.warn_message(
                        "can't delete %s cache directory." % v[0])
                    pass
                pass
            pass

        pass

    pass



# -*- coding: utf-8 -*-


from __future__ import absolute_import

import logging
import os

logger = logging.getLogger(__name__)


class QingCloud(object):
    """
    Wrapper for official qingcloud package
    """

    connector = None

    def __init__(self, zone=None):
        super(QingCloud, self).__init__()
        self.zone = zone
        self.logger = logger
        pass

    def connect(self, zone=None):
        """
        :return:
        """
        import qingcloud.iaas

        zone = zone or self.zone

        key = os.environ.get("QINGCLOUD_ACCESS_KEY")
        secret = os.environ.get("QINGCLOUD_ACCESS_SECRET")

        self.logger.debug("api-auth with key: %s,%s", key,
                          secret[0:2] + '*' * 10 + secret[-2:])

        if None in [zone, key, secret]:
            raise ValueError(
                "Please set `QINGCLOUD_ACCESS_KEY',`QINGCLOUD_ACCESS_SECRET',"
                "and make sure `zone' has set")

        self.connector = qingcloud.iaas.connect_to_zone(
            zone,
            key,
            secret
        )

    def __getattr__(self, name):
        """
        wrapper for qingcloud
        :param name:
        :return:
        """

        def wrapper(*args, **kwargs):
            return getattr(self.connector, name)(*args, **kwargs)

        if hasattr(self.connector, name) and callable(
                getattr(self.connector, name)):
            return wrapper

        raise AttributeError(name)

    _create_rules = [
        # device_type,describe api,create api,modify api
        ('loadbalancer', '',),
        ('loadbalancer-policy', '',),
    ]

    def create(self, device_type, name, arg_list=[], arg_dict={},
               overwrite=False):
        """
        create device

        :param device_type:
        :param name:
        :param overwrite:
        :return:
        """

        for rule in self._create_rules:

            if device_type == rule[0]:
                method = getattr(self.client, rule[1])

                pass

            pass

        pass

    def get_loadbalancer_policy(self, name):
        """
        try get loadbalancer policy device by name.
        :param policy_name:
        :return: policy dict, if not exist, raise ValueError
        """

        policies = self.connector.describe_loadbalancer_policies()
        pos_name_set = [(k, v['loadbalancer_policy_name']) for k, v in
                        enumerate(policies['loadbalancer_policy_set'])]
        pos_set, name_set = zip(*pos_name_set)

        try:
            pos = pos_set[name_set.index(name)]
            policy = policies['loadbalancer_policy_set'][pos]
        except (ValueError, IndexError):
            raise ValueError("%s not exist" % name)

        return policy

    def get_or_create_loadbalancer_policy(self, policy_name, *args, **kwargs):
        """

        :param policy_name:
        :param overwrite:
        :return:
        """

        try:
            policy = self.get_loadbalancer_policy(policy_name)
        except ValueError:
            self.connector.create_loadbalancer_policy(policy_name, *args,
                                                      **kwargs)
            policy = self.get_loadbalancer_policy(policy_name)
            pass
        return policy

    def get_loadbalancer_policy_rule(self, policy_id, name):
        """
        try get loadbalancer policy rule by name.
        :param policy_id: policy id
        :param name: rule name
        :return: dict, if not exist, raise ValueError
        """
        rules = self.connector.describe_loadbalancer_policy_rules(
            loadbalancer_policy=policy_id)

        pos_name_set = [(k, v['loadbalancer_policy_rule_name']) for k, v in
                        enumerate(rules['loadbalancer_policy_rule_set'])]
        pos_set, name_set = zip(*pos_name_set)

        try:
            pos = pos_set[name_set.index(name)]
            rule = rules['loadbalancer_policy_rule_set'][pos]
        except (ValueError, IndexError):
            raise ValueError("%s not exist" % name)

        return rule

    def get_or_add_loadbalancer_policy_rules(self, policy_id, rule, *args,
                                             **kwargs):
        try:
            rule = self.get_loadbalancer_policy_rule(policy_id, rule[
                'loadbalancer_policy_rule_name'])
        except ValueError:
            self.connector.add_loadbalancer_policy_rules(policy_id, [rule],
                                                         *args, **kwargs)
            rule = self.get_loadbalancer_policy_rule(policy_id, rule[
                'loadbalancer_policy_rule_name'])
            pass

        return rule

    def get_loadbalancer_backend(self, listener_id, name):
        """
        try get load balancer backend
        :param policy_name:
        :return: policy dict, if not exist, raise ValueError
        """

        backends = self.connector.describe_loadbalancer_backends(
            loadbalancer_listener=listener_id)
        pos_name_set = [(k, v['loadbalancer_backend_name']) for k, v in
                        enumerate(backends['loadbalancer_backend_set'])]

        try:
            pos_set, name_set = zip(*pos_name_set)
            pos = pos_set[name_set.index(name)]
            backend = backends['loadbalancer_backend_set'][pos]
        except (ValueError, IndexError):
            raise ValueError("%s not exist" % name)

        return backend

    def get_or_add_load_balancer_backends(self, loadbalancer_listener, backend,
                                          *args, **kwargs):
        try:
            backend = self.get_loadbalancer_backend(
                loadbalancer_listener,
                backend[
                    'loadbalancer_backend_name'])
        except ValueError:
            self.connector.add_backends_to_listener(
                loadbalancer_listener,
                [backend], *args, **kwargs)
            backend = self.get_loadbalancer_backend(
                loadbalancer_listener,
                backend[
                    'loadbalancer_backend_name'])
            pass

        return backend

    # def get_or_create_server_certicifate(self, loadbalancer_listener,
    #                                      backend,
    #                                      *args, **kwargs):
    #     try:
    #         backend = self.get_loadbalancer_backend(
    #             loadbalancer_listener,
    #             backend[
    #                 'loadbalancer_backend_name'])
    #     except ValueError:
    #         self.connector.add_backends_to_listener(
    #             loadbalancer_listener,
    #             [backend], *args, **kwargs)
    #         backend = self.get_loadbalancer_backend(
    #             loadbalancer_listener,
    #             backend[
    #                 'loadbalancer_backend_name'])
    #         pass
    #
    #     return backend
    pass



# -*- coding: utf-8 -*-

import argparse
import os

from cliez import conf
from cliez.parser import parse

from cabric import version

conf.COMPONENT_ROOT = os.path.dirname(__file__)
conf.GENERAL_ARGUMENTS = [
    (('--dir',),
     dict(nargs='?', default=os.getcwd(), help='set working directory')),
    (('--debug',), dict(action='store_true', help='open debug mode')),
    (('--verbose', '-v'), dict(action='count')),
    (('--env', '-s'), dict(nargs='?', default='beta', help='set environment')),
    (('--hosts-file',), dict(help='chose another hosts file to load')),
]
conf.EPILOG = 'You can submit issues at: https://www.github.com/nextoa/cabric'


def main():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=conf.EPILOG,
    )

    for v in conf.GENERAL_ARGUMENTS:
        parser.add_argument(*v[0], **v[1])

    parser.add_argument('--version','-V', action='version',
                        version='%(prog)s v{}'.format(version))

    # do nothing when no argument applied
    # parse(parser)

    parse(parser)
    pass


if __name__ == "__main__":
    main()
    pass



# -*- coding: utf-8 -*-

"""
Most of them copy form `cabric.main` directly.

`import cabric.main` will make script work not right.

some error like:

..code-block::

    Usage: fab [options] <command>
    [:arg1,arg2=val2,host=foo,hosts='h1;h2',...] ...

    main.py: error: no such option: --skip-enable-services

"""

import os
from optparse import OptionParser

from fabric import state
from fabric.state import env_options


def parse_options(args=[]):
    """
    Handle command-line options with optparse.OptionParser.

    Return list of arguments, largely for use in `parse_arguments`.
    """
    #
    # Initialize
    #

    parser = OptionParser(
        usage=("cabric bridge [options] <command>"
               "[:arg1,arg2=val2,host=foo,hosts='h1;h2',...] ..."))

    #
    # Define options that don't become `env` vars (typically ones which cause
    # Fabric to do something other than its normal execution, such as
    # --version)
    #

    # Display info about a specific command
    parser.add_option('-d', '--display',
                      metavar='NAME',
                      help="print detailed info about command NAME"
                      )

    # Control behavior of --list
    LIST_FORMAT_OPTIONS = ('short', 'normal', 'nested')
    parser.add_option('-F', '--list-format',
                      choices=LIST_FORMAT_OPTIONS,
                      default='normal',
                      metavar='FORMAT',
                      help="formats --list, choices: %s" % ", ".join(
                          LIST_FORMAT_OPTIONS)
                      )

    parser.add_option('-I', '--initial-password-prompt',
                      action='store_true',
                      default=False,
                      help="Force password prompt up-front"
                      )

    parser.add_option('--initial-sudo-password-prompt',
                      action='store_true',
                      default=False,
                      help="Force sudo password prompt up-front"
                      )

    # List Fab commands found in loaded fabfiles/source files
    parser.add_option('-l', '--list',
                      action='store_true',
                      dest='list_commands',
                      default=False,
                      help="print list of possible commands and exit"
                      )

    # Allow setting of arbitrary env vars at runtime.
    parser.add_option('--set',
                      metavar="KEY=VALUE,...",
                      dest='env_settings',
                      default="",
                      help="comma separated KEY=VALUE pairs"
                           " to set Fab env vars"
                      )

    # Like --list, but text processing friendly
    parser.add_option('--shortlist',
                      action='store_true',
                      dest='shortlist',
                      default=False,
                      help="alias for -F short --list"
                      )

    # Version number (optparse gives you --version but we have to do it
    # ourselves to get -V too. sigh)
    parser.add_option('-V', '--version',
                      action='store_true',
                      dest='show_version',
                      default=False,
                      help="show program's version number and exit"
                      )

    #
    # Add in options which are also destined to show up as `env` vars.
    #

    for option in env_options:
        parser.add_option(option)

    #
    # Finalize
    #

    # Return three-tuple of parser + the output from parse_args (opt obj, args)
    # Hack:make fab not accept any argument.
    opts, args = parser.parse_args(args=args)
    return parser, opts, args


def update_output_levels(show, hide):
    """
    Update state.output values as per given comma-separated list of key names.

    For example, ``update_output_levels(show='debug,warnings')`` is
    functionally equivalent to ``state.output['debug'] = True ;
    state.output['warnings'] = True``. Conversely, anything given to ``hide``
    sets the values to ``False``.
    """
    if show:
        for key in show.split(','):
            state.output[key] = True
    if hide:
        for key in hide.split(','):
            state.output[key] = False


def load_settings(path):
    """
    Take given file path and return dictionary of any key=value pairs found.

    Usage docs are in sites/docs/usage/fab.rst, in "Settings files."
    """
    if os.path.exists(path):
        settings = filter(lambda s: s and not s.startswith("#"),
                          open(path, 'r'))
        return dict((k.strip(), v.strip()) for k, _, v in
                    [s.partition('=') for s in settings])
    # Handle nonexistent or empty settings file
    return {}



# -*- coding: utf-8 -*-

import os

from cliez.components.init import InitComponent


class RenderComponent(InitComponent):
    """
    alias for cliez init component
    """
    exclude_global_option = True

    @classmethod
    def add_arguments(cls):
        """
        render project template variable.

        a legal template variable is named three underscore
         line as prefix and suffix.

        such like this: ___name___

        """
        return [
            (('--yes',), dict(action='store_true', help='clean .git repo')),
            (('--variable', '-s'),
             dict(nargs='+', help='set extra variable,format is name:value')),
            (('--skip-builtin',),
             dict(action='store_true', help='skip replace builtin variable')),

            (('--dir',), dict(nargs='?', default=os.getcwd(),
                              help='set working directory')),
            (('--debug',), dict(action='store_true', help='open debug mode')),
            (('--dry-run',), dict(action='store_true',
                                  help='print command instead execute it')),
            (('--verbose', '-v'), dict(action='count')),
        ]

    pass



# -*- coding: utf-8 -*-

import os

from cliez.component import Component


class PackageComponent(Component):
    # un-comment this can disable global options
    # exclude_global_option = True

    def run(self, options):

        if options.public:
            repo = 'pypi'
            pass
        else:
            repo = options.source
            pass

        if options.register is True:
            os.system('python setup.py register -r %s' % repo)
            pass

        os.system('python setup.py sdist upload -r %s' % repo)
        pass

    @classmethod
    def add_arguments(cls):
        """
        sub parser document
        """
        return [
            (('--register', '-r'),
             dict(action='store_true', help='register package to repo.')),
            (('--source', '-s'),
             dict(default='cabric', help='try load private repo.', )),
            (('--public',),
             dict(action='store_true', help='upload to pypi repo.')),
        ]
        pass

    pass



# -*- coding: utf-8 -*-

import os

from cliez.components.create import CreateComponent


class TouchComponent(CreateComponent):
    """
    alias for cliez create component
    """
    exclude_global_option = True

    @classmethod
    def add_arguments(cls):
        """
        create project form repo.support github,bitbucket and local repo.

        By default,cabric search repo orders is:

            - github
            - bitbucket
            - local


        if user define `--local` option. search local path first.

        if user define `--bitbucket`, search bitbucket first,
        then search github.

        ..note::

            * currently,we only support ssh mode when use bitbucket *

        """
        return [
            (('repo',), dict(help='repo path.')),
            (('name',), dict(nargs='?', default='', help='project name.')),
            (('--local',),
             dict(action='store_true', help='try load repo local path.')),
            (('--bitbucket',),
             dict(action='store_true', help='search bitbucket first.')),

            (('--dir',), dict(nargs='?', default=os.getcwd(),
                              help='set working directory')),
            (('--debug',), dict(action='store_true', help='open debug mode')),
            (('--dry-run',), dict(action='store_true',
                                  help='print command instead execute it')),
            (('--verbose', '-v'), dict(action='count')),
        ]

    pass



# -*- coding: utf-8 -*-

from distutils.core import setup

from setuptools import find_packages

import cabric

setup(
    name='cabric',
    version=cabric.version,
    packages=find_packages(exclude=["tests"]),
    url='https://github.com/nextoa/cabric',
    download_url='https://github.com/nextoa/cabric/tarball/master',
    license='http://opensource.org/licenses/MIT',
    install_requires=[
        'requests',
        'gitpython',
        'pycrypto',
        'fabric',
        'cliez==2.0.12',
    ],
    author='WANG WENPEI',
    author_email='wangwenpei@nextoa.com',
    description='Cabric,a deploy tool for CentOS,based on Fabric.',
    keywords='fabric,cabric',
    entry_points={
        'console_scripts': [
            'cab = cabric.main:main',
        ]
    },

)



# -*- coding: utf-8 -*-

import os

from fabric.context_managers import env
from fabric.decorators import task

from cabric.utils import parse_hosts


@task
def ez(curr, current_path='./config/fabric'):
    """

    :param curr: which env to use
    :param current_path: set load path

      ..note::
        if you use `ol` as value, cabric will translate `online`


    :return:
    """
    curr = 'online' if curr == 'ol' else curr
    env_file = os.path.join(current_path, curr + '.conf')
    env.use_ssh_config = True
    env.hosts, env_host_names = parse_hosts(env_file)
    pass



# -*- coding: utf-8 -*-


import os
import sys

from fabric.context_managers import cd as fabric_cd
from fabric.context_managers import lcd as fabric_lcd
from fabric.context_managers import settings as fabric_settings
from fabric.network import disconnect_all
from fabric.operations import local as fabric_local
from fabric.operations import run as fabric_run
from fabric.state import env_options, env
from fabric.state import output
from fabric.tasks import execute as fab_execute

from cabric.bridge import parse_options, update_output_levels, load_settings


# try:
#     from shlex import quote as cmd_quote
# except ImportError:
#     from pipes import quote as cmd_quote


def _is_network_error_ignored():
    return not env.use_exceptions_for['network'] and env.skip_bad_hosts


def get_roots(project):
    """
    get relate config root of cabric.

    ..note::

        if config root or sub root not exist. it will raise OSError

    :return: tuple package_root,stage_root,fabric_root
    """

    project = os.path.expanduser(project)

    root = os.path.join(project, 'config')

    if not os.path.exists(root):
        raise OSError("%s not exists or permission denied" % root)

    package_root = os.path.join(root, 'cabric')
    config_root = os.path.join(root, 'stage')
    fabric_root = os.path.join(root, 'fabric')

    roots = (package_root, config_root, fabric_root)

    for sub_root in roots:
        if not os.path.exists(sub_root):
            raise OSError("%s not exists or permission denied" % sub_root)
        pass

    return roots


def put(local_path, remote_path):
    """
    ..todo::

        this should be improved.
        it may ignore .* file

    :param local_path:
    :param remote_path:
    :return:
    """
    local_path = os.path.expanduser(local_path)

    if env.host_string:
        rsync_bin = fabric_local('which rsync', capture=True)
        rsync = os.path.basename(rsync_bin)

        if not rsync:
            raise IOError("rsync not exist.please install it.")

        tpl = str('%(bin)s -e ssh%(port_str)s%(recursive_str)s '
                  '%(local_path)s %(host)s:%(remote_path)s')
        port = int(env.host_string[
                   env.host_string.find(':') + 1:]) if env.host_string.find(
            ':') > -1 else 22

        host = env.host_string[
               0:env.host_string.find(':')] if env.host_string.find(
            ':') > -1 else env.host_string

        port_str = ' %s' % port
        recursive_str = ''

        if os.path.isdir(local_path):
            local_path = local_path.rstrip('/') + '/'
            remote_path = remote_path.rstrip('/') + '/'
            recursive_str = ' -r'
            pass

        cmd = tpl % {
            'bin': rsync,
            'local_path': local_path,
            'remote_path': remote_path,
            'recursive_str': recursive_str,
            'port_str': port_str if port != 22 else '',
            'host': host
        }
        fabric_local(cmd)
    else:
        fabric_local("cp -rf %s %s" % (local_path, remote_path))

    pass


def mirror_put(local_root, remote_path, validate=True):
    """
    mirror input

    use rsync instead fabric put.
    because fabric put is too slow.

    :param local_root:
    :param remote_path:
    :param validate:

    :return:
    """

    local_path = local_root + remote_path

    checks = [
        not os.path.exists(local_root),
        not os.path.exists(local_path),
    ]

    if validate:
        if True in checks:
            raise OSError("%s not exists" % local_path)

    # all path exists,then try upload or copy it
    if True not in checks:
        put(local_path, remote_path)

    pass


def parse_hosts(file):
    """
    :param file:
    :return: machines and names pairs
    """

    file = os.path.expanduser(file)

    with open(file, 'r') as fp:
        buffer = fp.read()

    host_list = buffer.strip("\n").split("\n")
    machines = []
    names = []
    for v in host_list[:]:
        addr = v.strip()
        if not addr or addr.find('#') == 0:
            continue

        if addr.find('##') > 0:
            pairs = addr.split("#")
            machines.append(pairs[0].strip())
            names.append(pairs[-1].strip())
            pass
        else:
            machines.append(addr.split("#")[0].strip())
            names.append(None)

    return machines, names


def bind_hosts(fabric_root, select_env, parallel=False, machine_config=None):
    """
    bind hosts from file
    :param fabric_root:
    :param select_env:
    :param parallel:
    :param machine_config:
    :return:
    """

    machine_config = machine_config or os.path.join(fabric_root,
                                                    select_env + '.conf')

    if not os.path.exists(machine_config):
        raise OSError("%s not exist." % machine_config)

    if not os.access(machine_config, os.R_OK):
        raise OSError("access %s permission denied." % machine_config)

    inner_options = ['-P'] if parallel else []

    parser, options, arguments = parse_options(inner_options)

    # Handle regular args vs -- args
    arguments = parser.largs
    remainder_arguments = parser.rargs

    for option in env_options:
        env[option.dest] = getattr(options, option.dest)

    env.hosts, env.host_names = parse_hosts(machine_config)
    env.use_ssh_config = True
    env.roledefs[select_env] = env.hosts

    for key in ['hosts', 'roles', 'exclude_hosts']:
        if key in env and isinstance(env[key], basestring):
            env[key] = env[key].split(',')

    # Handle output control level show/hide
    update_output_levels(show=options.show, hide=options.hide)
    env.update(load_settings(env.rcfile))

    for v in env.hosts:
        flag_postion = v.find('@')

        if flag_postion > -1:
            host = v[flag_postion + 1:]
        else:
            host = v
            pass
        known_host(host, local_mode=True, clean=False)
        pass

    return machine_config


def cd(path):
    if env.hosts:
        return fabric_cd(path)
    else:
        return fabric_lcd(path)
    pass


def run(cmd, user=None, capture=None, *args, **kwargs):
    """
    :param cmd: command to execute
    :param user: user mode will be ignore when we deploy on local machine.
    :param capture: in local machine, it will be True by default.
    :param args:
    :param kwargs:
    :return:
    """
    if env.hosts:
        if user:
            return fabric_run("su - %s -c '%s'" % (user, cmd))
        else:
            return fabric_run(cmd, *args, **kwargs)
    else:
        capture = True if capture is None else capture
        return fabric_local(cmd, capture=capture, *args, **kwargs)
    pass


def run_block(centos=None, mac=None, all=None, with_exception=True):
    """

    :param centos:
    :param macos:
    :param all:
    :return:
    """

    remote_os = get_platform()

    if remote_os == 'centos':
        if centos:
            centos()
        pass
    elif remote_os == 'mac':
        if mac:
            mac()
        pass
    else:
        if with_exception:
            raise NotImplemented("not support platform.")
        else:
            return False

    if all:
        all()

    return True


def execute(commands):
    results = []

    try:
        commands_to_run = [(v, [], {}, [], [], []) for v in commands]

        for name, args, kwargs, arg_hosts, arg_roles, arg_exclude_hosts \
            in commands_to_run:
            results.append(fab_execute(name, hosts=arg_hosts, roles=arg_roles,
                                       exclude_hosts=arg_exclude_hosts, *args,
                                       **kwargs))
            pass
    except SystemExit:  # a number of internal functions might raise this one.
        raise
    except KeyboardInterrupt:
        if output.status:
            sys.stderr.write("\nStopped.\n")
        sys.exit(1)
    except:
        sys.excepthook(*sys.exc_info())
        # we might leave stale threads if we don't explicitly exit()
        sys.exit(1)
    finally:
        disconnect_all()

    return results


def get_platform():
    remote_os = run('uname')
    remote_os = remote_os.lower().strip("\n")

    if remote_os == 'darwin':
        return 'mac'
    elif remote_os == 'linux':
        return 'centos'
        # not all os has this file
        distribute = run("gawk -F= '/^NAME/{print $2}' /etc/os-release")
        if 'centos' in distribute.lower():
            return 'centos'
        pass

    return 'unknown'


def get_home(user):
    with fabric_settings(warn_only=True):
        if run('cat /etc/passwd | grep "^%s:"' % user).failed:
            raise ValueError("User:%s not exists" % user)
        else:
            user_path = run("cat /etc/passwd |"
                            " grep '^%s:' |"
                            " awk -F ':' '{print $6}'" % user)
            return user_path
    pass


def get_git_host(address):
    host_begin = address.find('@')
    host_end = address.find(':')

    if host_begin <= 0 or host_end <= 0:
        raise ValueError('invalid git address to parse')

    return address[host_begin + 1:host_end].strip()


def get_repo():
    path = os.getcwd()
    if os.path.exists(os.path.join(path, '.git')):
        return fabric_local('git remote get-url origin --push', capture=True)


def exist_user(user):
    """
    check user exist or not
    :param user:
    :return:
    """

    with fabric_settings(warn_only=True):
        if run('cat /etc/passwd | grep ^%s:' % user).failed:
            return False
        else:
            return True

    pass


def exist_group(group):
    """
    check group exist or not
    :param group: group name
    :return:
    """

    with fabric_settings(warn_only=True):
        if run('cat /etc/group | grep ^%s:' % group).failed:
            return False
        else:
            return True

    pass


def known_host(address, user=None, local_mode=False, clean=True):
    """
    set ssh fingerprint
    :param address:domain or ip
    :param user:remote user name
    :param local_mode: set known_host for localhost,default is False
    :param clean: clean exist record,default is True
    """

    user_path = '~' if local_mode else get_home(user)
    command0 = "grep '%s ' %s/.ssh/known_hosts" % (address, user_path)
    command1 = 'ssh-keyscan %s >> %s/.ssh/known_hosts' % (address, user_path)
    command2 = 'sed -i -e "s/%s//g" %s/.ssh/known_hosts' % (address, user_path)

    commander = fabric_local if local_mode else run

    if user:
        command0 = 'su - %s -c "%s"' % (user, command0)
        command1 = 'su - %s -c "%s"' % (user, command1)
        command2 = 'su - %s -c "%s"' % (user, command2)
        pass

    with fabric_settings(warn_only=True):
        if commander(command0).failed:
            commander(command1)
        else:
            if clean:
                # clean old record
                commander(command2)
                commander(command1)
                pass
            pass
        pass
    pass


def current_machine(position):
    try:
        if env.host_string == env.hosts[position]:
            return True
    except IndexError:
        pass

    return False



# -*- coding: utf-8 -*-

import os

from fabric.api import *
from fabric.main import list_commands
from fabric.utils import _AttributeDict

ez_env = _AttributeDict({
    'debug': None,
    'group': None,
    'roles': None,
    'cloud': None,

    # cloud
    'cc': {
        'cloud_class': None,
        'processor': None,
        'connection': None,
        'sleep_time': None,
        'config': None,
        'actions': {},
    },
    'cloud_processor': None,
    'cloud_class': None,
    'cloud_handler': None,  # current cloud handle
    'cloud_hold_recyle': 3,  # sleep time when request jobs
    'debug': None,
})


def _pssh(file):
    """
    Parse pssh-like file
    :param file:
    """
    file = os.path.realpath(os.path.expanduser(file))

    with open(file, 'r') as fp:
        buffer = fp.read()

    host_list = buffer.strip("\n").split("\n")
    machines = []
    for addr in host_list[:]:
        addr.strip()
        if addr.find('#') == -1:
            machines.append(addr)

    return machines


def bind_hosts(curr, routes=None):
    """
    bind hosts from file
    :param curr:
    :param routes:
    :return:
    """

    if not routes:
        current_path = './config/fabric'

        files = os.listdir(current_path)

        routes = {}

        for f in files:

            if f.endswith('conf'):
                k = f.rsplit('.')[0]
                v = os.path.join(current_path, f)

                if k == 'online':
                    k = 'ol'
                    pass

                routes[k] = v

                pass

            pass

        pass

    ez_env.group = curr
    ez_env.roles = {}

    for k, v in routes.items():
        if k == curr:
            env.hosts = _pssh(v)
            env.use_ssh_config = True
            pass
        if k == 'online':
            k = 'ol'
            pass
        ez_env.roles[k] = _pssh(v)
        env.roledefs[k] = ez_env.roles[k]
        pass
    pass


def bind_cloud(cloud_options, cloud_class=None):
    ez_env.cloud = cloud_options

    if cloud_class:
        ez_env.cc['cloud_class'] = cloud_class

    pass


def debug(flag=None):
    """
    get or set debug flag
    :param flag:
    :return:
    """

    if flag is not None:
        ez_env.debug = flag

    return ez_env.debug


def help():
    # print("called!")

    commands = list_commands(None, "nested")

    support_type = ['cmd', 'cloud', 'classic', 'config', 'io', 'py', 'git']
    ignore = ['Dumper', 'Loader', 'bind_cloud', 'bind_hosts', 'dump',
              'dump_codes']
    origin = ['Available commands:']

    print(commands)

    # commands.split("\n")

    return
    #
    for c in commands_buff:

        compare = c.strip()

        print(compare)

        if compare in ignore:
            continue

        if compare in origin:
            print(c)
            continue

        # print("    " + c)
        pass

        #
        # if format_ == "short":
        # return _task_names(state.commands)
        # # Otherwise, handle more verbose modes
        # result = []
        # # Docstring at top, if applicable
        # if docstring:
        # trailer = "\n" if not docstring.endswith("\n") else ""
        # result.append(docstring + trailer)
        # header = COMMANDS_HEADER
        # if format_ == "nested":
        # header += NESTED_REMINDER
        # result.append(header + ":\n")
        # c = _normal_list() \
        #    if format_ == "normal" else _nested_list(state.commands)
        # result.extend(c)
        # return result
